@@ -1,23 +1,10 @@
 /*
- *   Copyright (C) 2006 Aaron Seigo <aseigo@kde.org>
- *   Copyright (C) 2007, 2009 Ryan P. Bitanga <ryan.bitanga@gmail.com>
- *   Copyright (C) 2008 Jordi Polo <mumismo@gmail.com>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2006 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2007, 2009 Ryan P. Bitanga <ryan.bitanga@gmail.com>
+    SPDX-FileCopyrightText: 2008 Jordi Polo <mumismo@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "runnermanager.h"
 
@@ -26,28 +13,27 @@
 #include <QCoreApplication>
 #include <QStandardPaths>
 #include <QDir>
-#include "krunner_debug.h"
+#include <QRegularExpression>
 
-#include <ksharedconfig.h>
-#include <kplugininfo.h>
-#include <kservicetypetrader.h>
+#include <KSharedConfig>
+#include <KServiceTypeTrader>
 #include <KPluginMetaData>
+#include <KConfigWatcher>
 
 #include <ThreadWeaver/DebuggingAids>
 #include <ThreadWeaver/Queue>
 #include <ThreadWeaver/Thread>
 
+#include <plasma/version.h>
+
 #include "dbusrunner_p.h"
 #include "runnerjobs_p.h"
-#include "plasma/pluginloader.h"
-#include <plasma/version.h>
 #include "querymatch.h"
+#include "krunner_debug.h"
 
 using ThreadWeaver::Queue;
 using ThreadWeaver::Job;
 
-//#define MEASURE_PREPTIME
-
 namespace Plasma
 {
 
@@ -115,17 +101,11 @@ public:
         QObject::connect(q, &RunnerManager::matchesChanged, q, [&] { lastMatchChangeSignalled.restart(); });
     }
 
-    ~RunnerManagerPrivate()
-    {
-        KConfigGroup config = configGroup();
-        context.save(config);
-    }
-
     void scheduleMatchesChanged()
     {
         if(lastMatchChangeSignalled.hasExpired(250)) {
             matchChangeTimer.stop();
-            emit q->matchesChanged(context.matches());
+            Q_EMIT q->matchesChanged(context.matches());
         } else {
             matchChangeTimer.start(250 - lastMatchChangeSignalled.elapsed());
         }
@@ -133,7 +113,7 @@ public:
 
     void matchesChanged()
     {
-        emit q->matchesChanged(context.matches());
+        Q_EMIT q->matchesChanged(context.matches());
     }
 
     void loadConfiguration()
@@ -144,13 +124,15 @@ public:
         // to half the number of threads
         DefaultRunnerPolicy::instance().setCap(qMax(2, Queue::instance()->maximumNumberOfThreads() / 2));
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
         enabledCategories = config.readEntry("enabledCategories", QStringList());
+#endif
         context.restore(config);
     }
 
     KConfigGroup configGroup()
     {
-        return conf.isValid() ? conf : KConfigGroup(KSharedConfig::openConfig(), "PlasmaRunnerManager");
+        return conf.isValid() ? conf : KConfigGroup(KSharedConfig::openConfig(configFile), "PlasmaRunnerManager");
     }
 
     void clearSingleRunner()
@@ -218,7 +200,7 @@ QT_WARNING_POP
             currentSingleRunner = loadInstalledRunner(pluginMetaData);
 
             if (currentSingleRunner) {
-                emit currentSingleRunner->prepare();
+                Q_EMIT currentSingleRunner->prepare();
                 singleRunnerWasLoaded = true;
             }
         }
@@ -235,7 +217,7 @@ QT_WARNING_POP
         if (conf.isValid()) {
             pluginConf = KConfigGroup(&conf, "Plugins");
         } else {
-            pluginConf = KConfigGroup(KSharedConfig::openConfig(), "Plugins");
+            pluginConf = KConfigGroup(KSharedConfig::openConfig(configFile), "Plugins");
         }
 
         advertiseSingleRunnerIds.clear();
@@ -270,20 +252,21 @@ QT_WARNING_POP
                     runner = loadInstalledRunner(description);
                 } else {
                     runner = runners.value(runnerName);
-                    runner->reloadConfiguration();
                 }
 
                 if (runner) {
+                    bool allCategoriesDisabled = true;
+                    #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
                     const QStringList categories = runner->categories();
                     allCategories << categories;
 
-                    bool allCategoriesDisabled = true;
                     for (const QString &cat : categories) {
                         if (enabledCategories.contains(cat)) {
                             allCategoriesDisabled = false;
                             break;
                         }
                     }
+                    #endif
 
                     if (enabledCategories.isEmpty() || !allCategoriesDisabled) {
                         qCDebug(KRUNNER) << "Loaded:" << runnerName;
@@ -342,9 +325,7 @@ QT_WARNING_POP
             clearSingleRunner();
         }
 
-#ifndef NDEBUG
-        // qCDebug(KRUNNER) << "All runners loaded, total:" << runners.count();
-#endif
+        qCDebug(KRUNNER) << "All runners loaded, total:" << runners.count();
     }
 
     AbstractRunner *loadInstalledRunner(const KPluginMetaData &pluginMetaData)
@@ -370,11 +351,6 @@ QT_WARNING_POP
                         QVariant::fromValue(pluginMetaData),
                     };
                     runner = factory->create<AbstractRunner>(q, args);
-                    if (!runner) {
-#ifndef NDEBUG
-                        // qCDebug(KRUNNER) << "Failed to load runner:" << pluginInfo.name() << ". error reported:" << pluginLoader.errorString();
-#endif
-                    }
                 } else {
                     qCWarning(KRUNNER).nospace() << "Could not load runner " << pluginMetaData.name() << ":"
                         << pluginLoader.errorString() << " (library path was:" << pluginMetaData.fileName() << ")";
@@ -387,18 +363,14 @@ QT_WARNING_POP
         } else if (api == QLatin1String("DBus")){
             runner = new DBusRunner(pluginMetaData, q);
         } else {
-            //qCDebug(KRUNNER) << "got a script runner known as" << api;
             runner = new AbstractRunner(pluginMetaData, q);
         }
 
         if (runner) {
-#ifndef NDEBUG
-            // qCDebug(KRUNNER) << "================= loading runner:" << pluginInfo.name() << "=================";
-#endif
             QObject::connect(runner, SIGNAL(matchingSuspended(bool)), q, SLOT(runnerMatchingSuspended(bool)));
             runner->init();
             if (prepped) {
-                emit runner->prepare();
+                Q_EMIT runner->prepare();
             }
         }
 
@@ -414,7 +386,6 @@ QT_WARNING_POP
         }
 
         if (deferredRun.isEnabled() && runJob->runner() == deferredRun.runner()) {
-            //qCDebug(KRUNNER) << "job actually done, running now **************";
             QueryMatch tmpRun = deferredRun;
             deferredRun = QueryMatch(nullptr);
             tmpRun.run(context);
@@ -427,17 +398,15 @@ QT_WARNING_POP
             // we finished our run, and there are no valid matches, and so no
             // signal will have been sent out. so we need to emit the signal
             // ourselves here
-            emit q->matchesChanged(context.matches());
+            Q_EMIT q->matchesChanged(context.matches());
         }
         if (searchJobs.isEmpty()) {
-            emit q->queryFinished();
+            Q_EMIT q->queryFinished();
         }
     }
 
     void checkTearDown()
     {
-        //qCDebug(KRUNNER) << prepped << teardownRequested << searchJobs.count() << oldSearchJobs.count();
-
         if (!prepped || !teardownRequested) {
             return;
         }
@@ -450,7 +419,7 @@ QT_WARNING_POP
         if (searchJobs.isEmpty() && oldSearchJobs.isEmpty()) {
             if (allRunnersPrepped) {
                 for (AbstractRunner *runner : qAsConst(runners)) {
-                    emit runner->teardown();
+                    Q_EMIT runner->teardown();
                 }
 
                 allRunnersPrepped = false;
@@ -458,7 +427,7 @@ QT_WARNING_POP
 
             if (singleRunnerPrepped) {
                 if (currentSingleRunner) {
-                    emit currentSingleRunner->teardown();
+                    Q_EMIT currentSingleRunner->teardown();
                 }
 
                 singleRunnerPrepped = false;
@@ -495,7 +464,11 @@ QT_WARNING_POP
 
     void startJob(AbstractRunner *runner)
     {
-        if ((runner->ignoredTypes() & context.type()) == 0) {
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
+        if ((runner->ignoredTypes() & context.type()) != 0) {
+            return;
+        }
+#endif
             QSharedPointer<FindMatchesJob> job(new FindMatchesJob(runner, &context, Queue::instance()));
             QObject::connect(job.data(), SIGNAL(done(ThreadWeaver::JobPointer)), q, SLOT(jobDone(ThreadWeaver::JobPointer)));
             if (runner->speed() == AbstractRunner::SlowSpeed) {
@@ -503,13 +476,12 @@ QT_WARNING_POP
             }
             Queue::instance()->enqueue(job);
             searchJobs.insert(job);
-        }
     }
 
     // Delay in ms before slow runners are allowed to run
     static const int slowRunDelay = 400;
 
-    RunnerManager *q;
+    RunnerManager *const q;
     QueryMatch deferredRun;
     RunnerContext context;
     QTimer matchChangeTimer;
@@ -530,6 +502,8 @@ QT_WARNING_POP
     bool singleMode : 1;
     bool singleRunnerWasLoaded : 1;
     QStringList whiteList;
+    QString configFile;
+    KConfigWatcher::Ptr watcher;
 };
 
 /*****************************************************
@@ -537,23 +511,27 @@ QT_WARNING_POP
 *
 *****************************************************/
 RunnerManager::RunnerManager(QObject *parent)
+    : RunnerManager(QString(), parent)
+{
+}
+
+RunnerManager::RunnerManager(const QString &configFile, QObject *parent)
     : QObject(parent),
       d(new RunnerManagerPrivate(this))
 {
+    d->configFile = configFile;
     d->loadConfiguration();
-    //ThreadWeaver::setDebugLevel(true, 4);
 }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 RunnerManager::RunnerManager(KConfigGroup &c, QObject *parent)
     : QObject(parent),
       d(new RunnerManagerPrivate(this))
 {
-    // Should this be really needed? Maybe d->loadConfiguration(c) would make
-    // more sense.
     d->conf = KConfigGroup(&c, "PlasmaRunnerManager");
     d->loadConfiguration();
-    //ThreadWeaver::setDebugLevel(true, 4);
 }
+#endif
 
 RunnerManager::~RunnerManager()
 {
@@ -580,6 +558,7 @@ void RunnerManager::setAllowedRunners(const QStringList &runners)
     }
 }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 void RunnerManager::setEnabledCategories(const QStringList& categories)
 {
     KConfigGroup config = d->configGroup();
@@ -591,6 +570,7 @@ void RunnerManager::setEnabledCategories(const QStringList& categories)
         d->loadRunners();
     }
 }
+#endif
 
 QStringList RunnerManager::allowedRunners() const
 {
@@ -598,6 +578,7 @@ QStringList RunnerManager::allowedRunners() const
     return config.readEntry("pluginWhiteList", QStringList());
 }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 QStringList RunnerManager::enabledCategories() const
 {
     KConfigGroup config = d->configGroup();
@@ -611,6 +592,7 @@ QStringList RunnerManager::enabledCategories() const
 
     return list;
 }
+#endif
 
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72) && KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
 void RunnerManager::loadRunner(const KService::Ptr service)
@@ -743,9 +725,6 @@ void RunnerManager::run(const QueryMatch &match)
 
     for (auto it = d->searchJobs.constBegin(); it != d->searchJobs.constEnd(); ++it) {
         if ((*it)->runner() == runner && !(*it)->isFinished()) {
-#ifndef NDEBUG
-            // qCDebug(KRUNNER) << "deferred run";
-#endif
             d->deferredRun = match;
             return;
         }
@@ -847,21 +826,12 @@ void RunnerManager::setupMatchSession()
     d->prepped = true;
     if (d->singleMode) {
         if (d->currentSingleRunner) {
-            emit d->currentSingleRunner->prepare();
+            Q_EMIT d->currentSingleRunner->prepare();
             d->singleRunnerPrepped = true;
         }
     } else {
         for (AbstractRunner *runner : qAsConst(d->runners)) {
-#ifdef MEASURE_PREPTIME
-            QTime t;
-            t.start();
-#endif
-            emit runner->prepare();
-#ifdef MEASURE_PREPTIME
-#ifndef NDEBUG
-            // qCDebug(KRUNNER) << t.elapsed() << runner->name();
-#endif
-#endif
+            Q_EMIT runner->prepare();
         }
 
         d->allRunnersPrepped = true;
@@ -876,6 +846,10 @@ void RunnerManager::matchSessionComplete()
 
     d->teardownRequested = true;
     d->checkTearDown();
+    // We save the context config after each session, just like the history entries
+    // BUG: 424505
+    KConfigGroup config = d->configGroup();
+    d->context.save(config);
 }
 
 void RunnerManager::launchQuery(const QString &term)
@@ -895,15 +869,6 @@ void RunnerManager::launchQuery(const QString &untrimmedTerm, const QString &run
         return;
     }
 
-    if (term.isEmpty()) {
-        if (d->singleMode && d->currentSingleRunner->defaultSyntax()) {
-            term = d->currentSingleRunner->defaultSyntax()->exampleQueries().first().remove(QLatin1String(":q:"));
-        } else {
-            reset();
-            return;
-        }
-    }
-
     if (d->context.query() == term) {
         // we already are searching for this!
         return;
@@ -914,27 +879,46 @@ void RunnerManager::launchQuery(const QString &untrimmedTerm, const QString &run
     }
 
     reset();
-//    qCDebug(KRUNNER) << "runners searching for" << term << "on" << runnerName;
     d->context.setQuery(term);
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
     d->context.setEnabledCategories(d->enabledCategories);
+#endif
 
-    QHash<QString, AbstractRunner*> runable;
+    QHash<QString, AbstractRunner*> runnable;
 
     //if the name is not empty we will launch only the specified runner
     if (d->singleMode) {
-        runable.insert(QString(), d->currentSingleRunner);
+        runnable.insert(QString(), d->currentSingleRunner);
         d->context.setSingleRunnerQueryMode(true);
     } else {
-        runable = d->runners;
+        runnable = d->runners;
     }
 
-    for (Plasma::AbstractRunner *r : qAsConst(runable)) {
+    const int queryLetterCount = term.count();
+    for (Plasma::AbstractRunner *r : qAsConst(runnable)) {
         if (r->isMatchingSuspended()) {
             continue;
         }
+        // The runners can set the min letter count as a property, this way we don't
+        // have to spawn threads just for the runner to reject the query, because it is too short
+        if (!d->singleMode && queryLetterCount < r->minLetterCount()) {
+            continue;
+        }
+        // If the runner has one ore more trigger words it can set the matchRegex to prevent
+        // thread spawning if the pattern does not match
+        if (!d->singleMode && r->hasMatchRegex() && !r->matchRegex().match(term).hasMatch()) {
+            continue;
+        }
 
         d->startJob(r);
     }
+    // In the unlikely case that no runner gets queried we have to emit the signals here
+    if (d->searchJobs.isEmpty()) {
+        QTimer::singleShot(0, this,  [this](){
+            Q_EMIT matchesChanged({});
+            Q_EMIT queryFinished();
+        });
+    }
 
     // Start timer to unblock slow runners
     d->delayTimer.start(RunnerManagerPrivate::slowRunDelay);
@@ -960,14 +944,40 @@ void RunnerManager::reset()
     d->searchJobs.clear();
 
     if (d->deferredRun.isEnabled()) {
-        //qCDebug(KRUNNER) << "job actually done, running now **************";
         QueryMatch tmpRun = d->deferredRun;
         d->deferredRun = QueryMatch(nullptr);
         tmpRun.run(d->context);
     }
 
     d->context.reset();
-    emit queryFinished();
+    Q_EMIT queryFinished();
+}
+
+void RunnerManager::enableKNotifyPluginWatcher()
+{
+    if (!d->watcher) {
+        d->watcher = KConfigWatcher::create(KSharedConfig::openConfig(d->configGroup().config()->name()));
+        connect(d->watcher.data(), &KConfigWatcher::configChanged, this, [this](const KConfigGroup &group, const QByteArrayList &changedNames) {
+            const QString groupName = group.name();
+            if (groupName == QLatin1String("Plugins")) {
+                reloadConfiguration();
+            } else if (groupName == QLatin1String("Runners")) {
+                for (auto *runner : qAsConst(d->runners)) {
+                    // Signals from the KCM contain the component name, which is the X-KDE-PluginInfo-Name property
+                    if (changedNames.contains(runner->metadata(RunnerReturnPluginMetaData).pluginId().toUtf8())) {
+                        runner->reloadConfiguration();
+                    }
+                }
+            } else if (group.parent().isValid() && group.parent().name() == QLatin1String("Runners")){
+                for (auto *runner : qAsConst(d->runners)) {
+                    // If the same config group has been modified which gets created in AbstractRunner::config()
+                    if (groupName == runner->id()) {
+                        runner->reloadConfiguration();
+                    }
+                }
+            }
+        });
+    }
 }
 
 } // Plasma namespace
