@@ -1,23 +1,10 @@
 /*
- *   Copyright (C) 2006 Aaron Seigo <aseigo@kde.org>
- *   Copyright (C) 2007 Ryan P. Bitanga <ryan.bitanga@gmail.com>
- *   Copyright (C) 2008 Jordi Polo <mumismo@gmail.com>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2006 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2007 Ryan P. Bitanga <ryan.bitanga@gmail.com>
+    SPDX-FileCopyrightText: 2008 Jordi Polo <mumismo@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef PLASMA_RUNNERMANAGER_H
 #define PLASMA_RUNNERMANAGER_H
@@ -25,7 +12,8 @@
 #include <QList>
 #include <QObject>
 
-#include <kplugininfo.h>
+#include <KPluginInfo>
+#include <KPluginMetaData>
 
 #include "krunner_export.h"
 #include "abstractrunner.h"
@@ -52,7 +40,14 @@ class KRUNNER_EXPORT RunnerManager : public QObject
 
     public:
         explicit RunnerManager(QObject *parent=nullptr);
+        explicit RunnerManager(const QString &configFile, QObject *parent=nullptr);
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 76)
+        /**
+         * @deprecated Since 5.76, use "RunnerManager(const QString &configFile, QObject *parent)" instead.
+         */
+        KRUNNER_DEPRECATED_VERSION(5, 76, "use RunnerManager(const QString &configFile, QObject *parent) instead")
         explicit RunnerManager(KConfigGroup &config, QObject *parent=nullptr);
+#endif
         ~RunnerManager();
 
         /**
@@ -157,20 +152,26 @@ class KRUNNER_EXPORT RunnerManager : public QObject
         void reloadConfiguration();
 
         /**
-         * Sets a whitelist for the plugins that can be loaded
+         * Sets a whitelist for the plugins that can be loaded by this manager.
          *
          * @param plugins the plugin names of allowed runners
          * @since 4.4
          */
         void setAllowedRunners(const QStringList &runners);
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 76)
         /**
          * Sets the list of categories which matches should be
          * returned for. It also internally tries not to execute the
          * runners which do not fall in this category.
+         * @deprecated Since 5.76, feature is unused and not supported by most runners
          */
+        KRUNNER_DEPRECATED_VERSION(5, 76, "feature is unused and not supported by most runners")
         void setEnabledCategories(const QStringList &categories);
+#endif
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 72)
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
         /**
          * Attempts to add the AbstractRunner plugin represented
          * by the KService passed in. Usually one can simply let
@@ -180,8 +181,24 @@ class KRUNNER_EXPORT RunnerManager : public QObject
          *
          * @param service the service to use to load the plugin
          * @since 4.5
+         * @deprecated Since 5.72, use loadRunner(const KPluginMetaData &)
          */
+        KRUNNER_DEPRECATED_VERSION(5, 72, "use loadRunner(const KPluginMetaData &)")
         void loadRunner(const KService::Ptr service);
+#endif
+#endif
+
+        /**
+         * Attempts to add the AbstractRunner plugin represented
+         * by the plugin info passed in. Usually one can simply let
+         * the configuration of plugins handle loading Runner plugins,
+         * but in cases where specific runners should be loaded this
+         * allows for that to take place
+         *
+         * @param pluginMetaData the metaData to use to load the plugin
+         * @since 5.72
+         */
+        void loadRunner(const KPluginMetaData &pluginMetaData);
 
         /**
          * Attempts to add the AbstractRunner from a Plasma::Package on disk.
@@ -200,10 +217,14 @@ class KRUNNER_EXPORT RunnerManager : public QObject
          */
         QStringList allowedRunners() const;
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 76)
         /**
          * @return the list of enabled categories
+         * @deprecated Since 5.76, feature is unused and not supported by most runners
          */
+        KRUNNER_DEPRECATED_VERSION(5, 76, "feature is unused and not supported by most runners")
         QStringList enabledCategories() const;
+#endif
 
         /**
          * @return mime data of the specified match
@@ -217,6 +238,20 @@ class KRUNNER_EXPORT RunnerManager : public QObject
          */
         QMimeData * mimeDataForMatch(const QString &matchId) const;
 
+        /**
+         * Returns a list of all known Runner implementations
+         *
+         * @param parentApp the application to filter runners on. Uses the
+         *                  X-KDE-ParentApp entry (if any) in the plugin metadata.
+         *                  The default value of QString() will result in a
+         *                  list containing only runners not specifically
+         *                  registered to an application.
+         * @return list of metadata of known runners
+         * @since 5.72
+         **/
+        static QVector<KPluginMetaData> runnerMetaDataList(const QString &parentApp = QString());
+
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 72)
         /**
          * Returns a list of all known Runner implementations
          *
@@ -227,8 +262,19 @@ class KRUNNER_EXPORT RunnerManager : public QObject
          *                  registered to an application.
          * @return list of AbstractRunners
          * @since 4.6
+         * @deprecated since 5.72, use runnerMetaDataList() instead
          **/
+        KRUNNER_DEPRECATED_VERSION(5, 72, "Use runnerMetaDataList() instead")
         static KPluginInfo::List listRunnerInfo(const QString &parentApp = QString());
+#endif
+
+        /**
+         * If you call this method the manager will create a KConfigWatcher
+         * which reload its runners or the runner configuration when the settings in the KCM are edited.
+         * @since 5.73
+         * @see reloadConfiguration
+         */
+        void enableKNotifyPluginWatcher();
 
     public Q_SLOTS:
         /**
